package com.yonge.cooleshow.admin.controller;

import com.baomidou.mybatisplus.core.metadata.IPage;

import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import com.yonge.toolset.utils.string.StringUtil;
import io.swagger.annotations.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.validation.Valid;

import com.yonge.cooleshow.biz.dal.vo.UserWithdrawalVo;
import com.yonge.cooleshow.biz.dal.dto.search.TeacherWithdrawalSearch;
import com.yonge.cooleshow.biz.dal.entity.UserWithdrawal;
import com.yonge.cooleshow.biz.dal.service.UserWithdrawalService;

@RestController
@RequestMapping("/userWithdrawal")
@Api(value = "用户账户提现表", tags = "用户账户提现表")
public class UserWithdrawalController extends BaseController {

    @Autowired
    private UserWithdrawalService userWithdrawalService;

	/**
     * 查询单条
     */
    @GetMapping("/detail/{id}")
    @ApiOperation(value = "详情", notes = "传入id")
    public HttpResponseResult<UserWithdrawalVo> detail(@PathVariable("id") Long id) {
    	return succeed(userWithdrawalService.detail(id));
	}
    
    /**
     * 查询分页
     */
    @PostMapping("/page")
    @ApiOperation(value = "查询分页", notes = "传入teacherWithdrawalSearch")
    public HttpResponseResult<PageInfo<UserWithdrawalVo>> page(@RequestBody TeacherWithdrawalSearch query) {
		IPage<UserWithdrawalVo> pages = userWithdrawalService.selectPage(PageUtil.getPage(query), query);
        return succeed(PageUtil.pageInfo(pages));
	}
    
    /**
	 * 新增
	 */
	@PostMapping("/save")
	@ApiOperation(value = "新增", notes = "传入teacherWithdrawal")
	public HttpResponseResult save(@Valid @RequestBody UserWithdrawal userWithdrawal) {
    	return status(userWithdrawalService.save(userWithdrawal));
	}
    
    /**
	 * 修改
	 */
	@PostMapping("/update")
	@ApiOperation(value = "修改", notes = "传入teacherWithdrawal")
	public HttpResponseResult update(@Valid @RequestBody UserWithdrawal userWithdrawal) {
        return status(userWithdrawalService.updateById(userWithdrawal));
	}
    
    /**
	 * 新增或修改
	 */
    @PostMapping("/submit")
    @ApiOperation(value = "新增或修改", notes = "传入teacherWithdrawal")
	public HttpResponseResult submit(@Valid @RequestBody UserWithdrawal userWithdrawal) {
        return status(userWithdrawalService.saveOrUpdate(userWithdrawal));
    }

 	/**
	 * 删除
	 */
	@PostMapping("/remove")
	@ApiOperation(value = "逻辑删除", notes = "传入ids")
	public HttpResponseResult remove(@ApiParam(value = "主键集合", required = true) @RequestParam String ids) {
        if (StringUtil.isEmpty(ids)) {
			return failed("参数不能为空");
		}
		return status(userWithdrawalService.removeByIds(StringUtil.toLongList(ids)));
	}
}