package com.yonge.cooleshow.admin.controller;

import com.yonge.cooleshow.biz.dal.dto.search.VideoLessonSearch;
import com.yonge.cooleshow.biz.dal.entity.VideoLessonGroup;
import com.yonge.cooleshow.biz.dal.entity.VideoLessonGroupDetail;
import com.yonge.cooleshow.biz.dal.service.VideoLessonGroupDetailService;
import com.yonge.cooleshow.biz.dal.service.VideoLessonGroupService;
import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.biz.dal.valid.AddGroup;
import com.yonge.cooleshow.biz.dal.valid.SelectGroup;
import com.yonge.cooleshow.biz.dal.vo.VideoLessonPurchaseVo;
import com.yonge.cooleshow.biz.dal.vo.VideoLessonStudentDetailVo;
import com.yonge.cooleshow.biz.dal.vo.VideoLessonTeacherVo;
import com.yonge.cooleshow.common.controller.BaseController;
import com.yonge.cooleshow.common.entity.HttpResponseResult;
import com.yonge.cooleshow.common.page.PageInfo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.validation.constraints.NotNull;

/**
 * @Author: cy
 * @Date: 2022/4/2
 */
@RestController
@RequestMapping("/videoLesson")
@Api(tags = "视频课")
@Validated
public class VideoLessonController extends BaseController {
    @Autowired
    private VideoLessonGroupService lessonGroupService;
    @Autowired
    private VideoLessonGroupDetailService videoLessonGroupDetailService;

    @ApiOperation(value = "根据视频课组id查详情")
    @GetMapping(value = "/selectGroupById")
    public HttpResponseResult<VideoLessonGroup> selectGroupById(@NotNull Long groupId) {
        return succeed(lessonGroupService.selectGroup(groupId));
    }

    /**
     * @Description: 学员详情-查询视频课列表
     * @Author: cy
     * @Date: 2022/4/2
     */
    @ApiOperation(value = "学员详情-查询视频课列表")
    @PostMapping(value = "/student")
    public HttpResponseResult<PageInfo<VideoLessonStudentDetailVo>> selectStudentPage(@Validated(AddGroup.class) @RequestBody VideoLessonSearch search) {
        return succeed(PageUtil.pageInfo(lessonGroupService.selectStudentPage(PageUtil.getPage(search), search)));
    }

    /**
     * @Description: 老师详情-查询视频课列表
     * @Author: cy
     * @Date: 2022/4/6
     */
    @ApiOperation(value = "老师详情-查询视频课列表")
    @PostMapping(value = "/teacher")
    public HttpResponseResult<PageInfo<VideoLessonTeacherVo>> selectTeacherPage(@Validated(AddGroup.class) @RequestBody VideoLessonSearch search) {
        return succeed(PageUtil.pageInfo(lessonGroupService.selectTeacherPage(PageUtil.getPage(search), search)));
    }

    /**
     * @Description: 老师详情-视频课详情-查询购买学员列表
     * @Author: cy
     * @Date: 2022/4/6
     */
    @ApiOperation(value = "老师详情-视频课详情-查询购买学员列表", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/teacher/purchaseStudent", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<PageInfo<VideoLessonPurchaseVo>> selectPurchaseStudent(@Validated(SelectGroup.class) @RequestBody VideoLessonSearch search) {
        return succeed(PageUtil.pageInfo(lessonGroupService.selectPurchaseStudent(PageUtil.getPage(search), search)));
    }

    /**
     * @Description: 老师详情-视频课详情-教学计划
     * @Author: cy
     * @Date: 2022/4/6
     */
    @ApiOperation(value = "老师详情-视频课详情-教学计划", httpMethod = "POST", consumes = "application/json", produces = "application/json")
    @PostMapping(value = "/teacher/plan", consumes = "application/json", produces = "application/json")
    public HttpResponseResult<PageInfo<VideoLessonGroupDetail>> page(@Validated(SelectGroup.class) @RequestBody VideoLessonSearch search) {
        return succeed(PageUtil.pageInfo(videoLessonGroupDetailService.selectPage(PageUtil.getPage(search), search)));
    }
}
