package com.yonge.cooleshow.biz.dal.dao;

import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yonge.cooleshow.biz.dal.dto.search.MyCourseSearch;
import com.yonge.cooleshow.biz.dal.dto.search.VideoLessonGroupSearch;
import com.yonge.cooleshow.biz.dal.entity.CourseSchedule;
import com.yonge.cooleshow.biz.dal.vo.MyCourseVo;
import com.yonge.cooleshow.biz.dal.vo.StudentTotalVo;
import com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo;
import com.yonge.cooleshow.biz.dal.vo.VideoLessonGroupVo;
import org.apache.ibatis.annotations.Param;

import java.util.List;
import java.util.Map;

/**
 * 老师课程表(CourseSchedule)表数据库访问层
 *
 * @author hgw
 * @since 2022-03-18 15:29:11
 */
public interface CourseScheduleDao extends BaseMapper<CourseSchedule> {

    int insertBatch(@Param("entities") List<CourseSchedule> entities);

    /***
     * 查询所有老师课时统计
     * @author liweifan
     * @updateTime 2022/3/28 10:53
     * @return: com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo
     */
    List<TeacherTotalVo> queryTeacherHomeTotal();

    /***
     * 查询所有学生课时统计
     * @author liweifan
     * @updateTime 2022/3/28 10:53
     * @return: com.yonge.cooleshow.biz.dal.vo.TeacherTotalVo
     */
    List<StudentTotalVo> queryStudentHomeTotal();

    /**
     * 根据学生id查询大于当前时间并未开始的课程
     *
     * @param studentId 学生id
     * @param classDate 年月日
     */
    List<CourseSchedule> queryStudentCourse(@Param("studentId") Long studentId, @Param("classDate") String classDate);

    /**
     * 查询老师直播课程
     *
     * @param param 传入参数
     *              <p> - teacherId 老师id
     *              <p> - status 课程状态 NOT_START未开始 ING进行中 COMPLETE已完成
     *              <p> - type  PRACTICE 陪练课 LIVE 直播课
     *              <p> - startDate 开始时间
     *              <p> - endDate 结束时间
     *              <p> - subjectId 声部id
     */
    <T> IPage<T> queryLiveTeacherCourse(Page<T> page, @Param("param") Map<String, Object> param);

    //根据老师id查询购课学员
    List<MyCourseVo> selectPage(IPage page, @Param("param") MyCourseSearch query);
}

