package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.auth.api.enums.SysUserType;
import com.yonge.cooleshow.biz.dal.dao.MusicSheetDao;
import com.yonge.cooleshow.biz.dal.dto.MusicSheetDto;
import com.yonge.cooleshow.biz.dal.dto.req.TeacherMusicSheetAuditReq;
import com.yonge.cooleshow.biz.dal.dto.search.*;
import com.yonge.cooleshow.biz.dal.entity.*;
import com.yonge.cooleshow.biz.dal.enums.*;
import com.yonge.cooleshow.biz.dal.service.*;
import com.yonge.cooleshow.biz.dal.support.PageUtil;
import com.yonge.cooleshow.biz.dal.vo.*;
import com.yonge.cooleshow.common.exception.BizException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * MusicSheetService服务实现类
 * @author yzp
 * @date 2022-03-26 00:01:37
 * @version v1.0
 **/
@Service
public class MusicSheetServiceImpl extends ServiceImpl<MusicSheetDao,MusicSheet> implements MusicSheetService {

    private static final  Logger log = LoggerFactory.getLogger(MusicSheetServiceImpl.class);

    @Autowired
    private MusicSheetDao musicSheetDao;

    @Autowired
    private MusicSheetAccompanimentService musicSheetAccompanimentService;

    @Autowired
    private MusicFavoriteService musicFavoriteService;

    @Autowired
    private MusicSheetPurchaseRecordService musicSheetPurchaseRecordService;

    @Autowired
    private MusicSheetPracticeRecordService musicSheetPracticeRecordService;

    @Autowired
    private StudentService studentService;

    @Autowired
    private MusicSheetAuthRecordService musicSheetAuthRecordService;

    public MusicSheetDao getDao() {
        return musicSheetDao;
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveMp3AndAccompaniment(MusicSheetDto musicSheetDto, SysUser sysUser) {
        MusicSheet musicSheet = new MusicSheet();
        BeanUtils.copyProperties(musicSheetDto, musicSheet);
        //保存主表
        musicSheet.setCreateBy(sysUser.getId());
        musicSheet.setState(YesOrNoEnum.NO);
        musicSheet.setCreateTime(new Date());
        int count = musicSheetDao.insert(musicSheet);

        Long sheetId = musicSheet.getId();
        List<MusicSheetAccompaniment> list = musicSheetDto.getBackground();
        list.forEach(musicSheetAccompaniment -> musicSheetAccompaniment.setMusicSheetId(sheetId).setCreateBy(sysUser.getId()).setCreateTime(new Date()));
        musicSheetAccompanimentService.saveBatch(list);
        count += list.size();

        return count > 1;
    }

    @Override
    public IPage<MusicSheetVo> selectPage(IPage<MusicSheetVo> page, MusicSheetSearch query) {
        return page.setRecords(baseMapper.selectPage(page, query));

    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean state(Long id) {
        MusicSheet musicSheet = this.getById(id);
        if (musicSheet == null) {
            throw  new BizException("未找到曲目信息");
        }
        if (YesOrNoEnum.NO.getCode().equals(musicSheet.getState().getCode())) {
            musicSheet.setState(YesOrNoEnum.YES);
        } else {
            musicSheet.setState(YesOrNoEnum.NO);
        }
        return this.updateById(musicSheet);

    }

    @Override
    public IPage<MusicSheet> selectAlbumDetailPage(IPage<MusicSheet> page, MusicAlbumDetailSearch query) {
        return page.setRecords(baseMapper.selectAlbumDetailPage(page, query));
    }

    @Override
    public MusicSheetShareVo shareMusicSheet(SysUser sysUser) {
        MusicSheetSearch query = new MusicSheetSearch();
        // 设置只查老师用户
        query.setCreateBy(sysUser.getId());
        query.setAuditStatus(AuthStatusEnum.PASS);
        query.setState(YesOrNoEnum.YES);
        // 只查3条
        query.setRows(3);
        IPage<MusicSheetVo> musicSheetVoIPage = selectPage(PageUtil.getPage(query), query);
        MusicSheetShareVo musicSheetShareVo = new MusicSheetShareVo();
        musicSheetShareVo.setMusicSheetCount(musicSheetVoIPage.getTotal());
        musicSheetShareVo.setMusicSheetVoList(musicSheetVoIPage.getRecords());
        musicSheetShareVo.setName(sysUser.getUsername());
        musicSheetShareVo.setAvatar(sysUser.getAvatar());
        return musicSheetShareVo;
    }

    @Override
    public MusicSheetDetailVo detail(Long id, SysUser sysUser,SysUserType userType) {
        MusicSheetDetailVo detail = baseMapper.detail(id);
        if (detail == null) {
            throw new BizException("未找到曲目信息");
        }
        if (ChargeTypeEnum.FREE.getCode().equals(detail.getChargeType().getCode())) {
            detail.setPlay(YesOrNoEnum.YES);
        }  else if (SysUserType.STUDENT.getCode().equals(userType.getCode())) {
            detail.setPlay(YesOrNoEnum.NO);
            // 单曲购买 判断购买记录，有记录课播放
            if (ChargeTypeEnum.CHARGE.getCode().equals(detail.getChargeType().getCode())) {
                // 学生须判断是否能播放曲目
                boolean b = musicSheetPurchaseRecordService.checkPurchase(sysUser.getId(),detail.getId());
                if (b) {
                    detail.setPlay(YesOrNoEnum.YES);
                }
            } else if  (ChargeTypeEnum.VIP.getCode().equals(detail.getChargeType().getCode())) {
                // 会员 判断是否为会员， 会员可播放
                Student student = studentService.getById(sysUser.getId());
                if (new Date().compareTo(student.getMembershipEndTime()) < 0) {
                    detail.setPlay(YesOrNoEnum.YES);
                }
            }
            // 学生进入云教练练习，添加一条练习记录
            musicSheetPracticeRecordService.addRecord(id,sysUser.getId());
        } else {
            detail.setPlay(YesOrNoEnum.YES);
        }
        return detail;
    }

    @Override
    public IPage<MusicSheetVo> selectStudentPage(IPage<MusicSheetVo> page, StudentMusicSheetSearch query) {
        return page.setRecords(baseMapper.selectStudentMusicPage(page, query));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean setFavorite(Long userId, Long musicSheetId) {
        List<MusicFavorite> musicFavorites = musicFavoriteService.getFavorite(userId, musicSheetId);
        if (CollectionUtils.isEmpty(musicFavorites)) {
            MusicFavorite musicFavorite = new MusicFavorite();
            musicFavorite.setMusicSheetId(musicSheetId);
            musicFavorite.setUserId(userId);
            musicFavorite.setFavoriteTime(new Date());
            return musicFavoriteService.save(musicFavorite);
        } else {
            List<Long> ids = musicFavorites.stream().map(MusicFavorite::getId).collect(Collectors.toList());
            return musicFavoriteService.removeByIds(ids);
        }
    }

    @Override
    public IPage<MusicSheetVo> myMusic(IPage<MusicSheetVo> page, StudentMusicSheetSearch query) {
        return page.setRecords(baseMapper.selectMyMusicPage(page,query));
    }

    @Override
    public IPage<MusicSheetVo> favoriteMusic(IPage<MusicSheetVo> page, StudentMusicSheetSearch query) {
        return page.setRecords(baseMapper.selectFavoriteMusicPage(page,query));
    }

    @Override
    public IPage<MusicSheetVo> practiceMusic(IPage<MusicSheetVo> page, StudentMusicSheetSearch query) {
        // 预计最近练习表数据量会很多
        // 先找到最近练习的曲目id 在根据曲目id进行精确查找
        IPage<Long> practiceMusicIdPage = musicSheetPracticeRecordService.selectPracticeMusic(query,page.getPages(),page.getSize());
        List<Long> practiceMusicIdList = practiceMusicIdPage.getRecords();
        if (CollectionUtils.isEmpty(practiceMusicIdList)) {
            page.setRecords(new ArrayList<>());
            return page;
        }
        // 构建分页信息
        page.setRecords(baseMapper.selectPracticeMusicPage(query,practiceMusicIdList));
        page.setPages(practiceMusicIdPage.getPages());
        page.setCurrent(practiceMusicIdPage.getCurrent());
        page.setSize(practiceMusicIdPage.getSize());
        page.setTotal(practiceMusicIdPage.getTotal());
        return page;
    }

    @Override
    public IPage<StudentMusicSheetOrderVo> selectStudentOrderPage(IPage<StudentMusicSheetOrderVo> page, StudentMusicSheetOrderSearch query) {
        return page.setRecords(baseMapper.selectStudentOrderPage(page,query));
    }

    @Override
    public IPage<TeacherMusicSheetVo> selectTeacherPage(IPage<TeacherMusicSheetVo> page, TeacherMusicSheetSearch query) {
        return page.setRecords(baseMapper.selectTeacherPage(page,query));
    }

    @Override
    public IPage<TeacherMusicSheetVo> selectAuditPage(IPage<TeacherMusicSheetVo> page, TeacherMusicSheetAuditSearch query) {
        return page.setRecords(baseMapper.selectAuditPage(page,query));
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean audit(TeacherMusicSheetAuditReq param, Long userId) {

        MusicSheet musicSheet = this.getById(param.getMusicSheetId());
        if (musicSheet == null) {
            throw  new BizException("未找到曲目信息");
        }

        if (musicSheetAuthRecordService.audit(param,userId)) {
            musicSheet.setAuditStatus(param.getAuthStatus());
            return this.updateById(musicSheet);
        } else {
            return false;
        }
    }

    @Override
    public boolean del(Long id) {
        return this.lambdaUpdate()
                .eq(MusicSheet::getId,id)
                .set(MusicSheet::getDelFlag,true)
                .update();

    }
}