package com.yonge.cooleshow.biz.dal.service.impl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.yonge.cooleshow.auth.api.entity.SysUser;
import com.yonge.cooleshow.common.constant.SysConfigConstant;
import com.yonge.cooleshow.biz.dal.dao.CourseScheduleDao;
import com.yonge.cooleshow.biz.dal.dao.StudentStarDao;
import com.yonge.cooleshow.biz.dal.dto.search.StudentSearch;
import com.yonge.cooleshow.biz.dal.enums.CacheNameEnum;
import com.yonge.cooleshow.biz.dal.service.SysConfigService;
import com.yonge.cooleshow.biz.dal.vo.*;
import com.yonge.cooleshow.common.enums.UserGenderEnum;
import com.yonge.toolset.utils.date.DateUtil;
import com.yonge.toolset.utils.string.StringUtil;
import org.apache.commons.beanutils.BeanUtils;
import org.redisson.api.RedissonClient;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.yonge.cooleshow.biz.dal.entity.Student;
import com.yonge.cooleshow.biz.dal.dao.StudentDao;
import com.yonge.cooleshow.biz.dal.service.StudentService;
import org.springframework.util.CollectionUtils;

import java.lang.reflect.InvocationTargetException;
import java.util.*;
import java.util.stream.Collectors;


@Service
public class StudentServiceImpl extends ServiceImpl<StudentDao, Student> implements StudentService {
    @Autowired
    private StudentStarDao studentStarDao;
    @Autowired
    private CourseScheduleDao courseScheduleDao;
    @Autowired
    private SysConfigService sysConfigService;
    @Autowired
    private RedissonClient redissonClient;

    @Override
    public StudentVo detail(Long userId) {
        return baseMapper.detail(userId);
    }

    @Override
    public IPage<StudentVo> selectPage(IPage<StudentVo> page, StudentSearch studentSearch) {
        return page.setRecords(baseMapper.selectPage(page, studentSearch));
    }

    @Override
    public List<StudentTotalVo> queryStudentHomeTotal() {
        List<Student> students = baseMapper.selectList(Wrappers.<Student>emptyWrapper()
                .select("userId"));
        //查询关注老师数
        List<StudentTotalVo> studentStarTotal = studentStarDao.queryStudentHomeTotal();
        Map<Long, StudentTotalVo> studentStarMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(studentStarTotal)) {
            studentStarMap = studentStarTotal.stream().collect(Collectors.toMap(StudentTotalVo::getUserId, o -> o));
        }

        //查询已上未上课时数
        List<StudentTotalVo> studentCourseTotal = courseScheduleDao.queryStudentHomeTotal();
        Map<Long, StudentTotalVo> studentCourseMap = new HashMap<>();
        if (!CollectionUtils.isEmpty(studentCourseTotal)) {
            studentCourseMap = studentCourseTotal.stream().collect(Collectors.toMap(StudentTotalVo::getUserId, o -> o));
        }
        List<StudentTotalVo> resultList = new ArrayList<>();
        for (Student student : students) {
            StudentTotalVo totalVo = new StudentTotalVo();
            totalVo.setUserId(student.getUserId());
            StudentTotalVo starTotal = studentStarMap.get(student.getUserId());
            if (null != starTotal) {
                totalVo.setStarTeacherNum(starTotal.getStarTeacherNum());
            }
            StudentTotalVo courseTotal = studentCourseMap.get(student.getUserId());
            if (null != courseTotal) {
                totalVo.setFinshClassHours(courseTotal.getFinshClassHours());
                totalVo.setUnfinshClassHours(courseTotal.getUnfinshClassHours());
            }
            //todo 缺少累计练习天数 累计练习时长 累计评测次数
            resultList.add(totalVo);
            redissonClient.getBucket(CacheNameEnum.STUDENT_HOME_TOTAL.getRedisKey(totalVo.getUserId()))
                    .set(totalVo);
        }
        return resultList;
    }

    @Override
    public StudentHomeVo queryUserInfo(SysUser user) {
        Student student = getById(user.getId());
        StudentHomeVo studentHomeVo = new StudentHomeVo();
        try {
            BeanUtils.copyProperties(studentHomeVo, student);
        } catch (IllegalAccessException e) {
            e.printStackTrace();
        } catch (InvocationTargetException e) {
            e.printStackTrace();
        }
        studentHomeVo.setHeardUrl(user.getAvatar());
        studentHomeVo.setUsername(user.getUsername());
        int num = DateUtil.daysBetween(new Date(), student.getMembershipEndTime());
        studentHomeVo.setMembershipDays(num < 0 ? 0 : num);
        StudentTotalVo totalVo = (StudentTotalVo) redissonClient.getBucket(CacheNameEnum.STUDENT_HOME_TOTAL.getRedisKey(user.getId()))
                .get();
        if (null == totalVo) {
            totalVo = new StudentTotalVo();
        }
        studentHomeVo.setExerciseDays(null == totalVo.getExerciseDays() ? 0 : totalVo.getExerciseDays());
        studentHomeVo.setExerciseHours(null == totalVo.getExerciseHours() ? 0 : totalVo.getExerciseHours());
        studentHomeVo.setTestingNum(null == totalVo.getTestingNum() ? 0 : totalVo.getTestingNum());
        studentHomeVo.setFinshClassHours(null == totalVo.getFinshClassHours() ? 0 : totalVo.getFinshClassHours());
        studentHomeVo.setUnfinshClassHours(null == totalVo.getUnfinshClassHours() ? 0 : totalVo.getUnfinshClassHours());
        studentHomeVo.setStarTeacherNum(null == totalVo.getStarTeacherNum() ? 0 : totalVo.getStarTeacherNum());
        return studentHomeVo;
    }

    @Override
    public Integer setSubject(Long id, String subjectIds) {
        return baseMapper.setSubject(id,subjectIds);
    }
}